# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"DeepWord": "00_core.ipynb",
         "URLs": "01_utils.ipynb",
         "TextDicts": "01_utils.ipynb",
         "to_hhmmss": "01_utils.ipynb",
         "to_secs": "01_utils.ipynb",
         "display_video": "01_utils.ipynb",
         "display_audio": "01_utils.ipynb",
         "trim_video": "01_utils.ipynb",
         "check_resolution": "01_utils.ipynb",
         "check_fps": "01_utils.ipynb",
         "change_audio_format": "01_utils.ipynb",
         "trim_audio": "01_utils.ipynb"}

modules = ["core.py",
           "utils.py"]

doc_url = "https://deepword18.github.io/dword/"

git_url = "https://github.com/deepword18/dword/tree/master/"

def custom_doc_links(name): return None
